//XmlNode.cpp
#ifndef _XMLNODE_H_
#include "XmlNode.h"
int XmlNode::SEQUENCE_SIGN=0; 

XmlNode::~XmlNode()
{
	if(attribute!=NULL)
	{
		delete attribute;
		attribute=NULL;
	} 
	if(childNodes!=NULL)
	{
		delete childNodes;
		childNodes=NULL;
	}
}

void XmlNode::SetAttribute(XmlAttribute* attr)
{
	if(attribute!=NULL && attribute!=attr)
	{ 
		delete attribute; 
	}
	attribute=attr;
}

void XmlNode::AppendAttr(const string& name,const string& value)
{
	if(attribute==NULL)
	{
		attribute=new XmlAttribute();
	} 
	attribute->Append(name,value);
}

XmlAttribute* XmlNode::Attribute() const
{
	return attribute;
}

int XmlNode::AttrCount() const
{
	if(attribute==NULL)
		return 0;
	return attribute->Count();
}

void XmlNode::AppendNode(XmlNode* node)
{
	if(childNodes==NULL)
	{
		childNodes=new XmlNodeList(); 
	}
	childNodes->AppendNode(node);
}

XmlNode* XmlNode::FirstChild()
{
	if(childNodes==NULL || childNodes->Count()==0) 
		return NULL;
	return (*childNodes)[0];
}

XmlNode* XmlNode::LastChild()
{
	if(childNodes==NULL || childNodes->Count()==0) 
		return NULL;
		return (*childNodes)[childNodes->Count()-1]; 
}

int XmlNode::ChildCount() const
{
	if(childNodes==NULL)
		return 0;
	return childNodes->Count();
}

XmlNodeList* XmlNode::ChildNodes() const
{
	return childNodes;
}

string XmlNode::Value() const
{
	return value;
}

void XmlNode::SetValue(const string& _value)
{
	this->value=_value;
}

string XmlNode::Name() const
{
	return name;
}

void XmlNode::SetName(const string& _name)
{
	this->name=_name;
}
 
XmlNode* XmlNode::Parent() const
{
	return parent;	
}

void XmlNode::ShowXML(XmlNode& node)const
{   
	if(node.ChildCount()>0)
	{
		cout<<endl;
	}
	if(node.name!=string()){
		cout<< "<"<<node.name;
	}
	ShowAttr();
	if(node.name!=string()){
		cout<<">";  
	}
	if(node.childNodes!=NULL)
	{ 
		XmlNodeList::Iterator iter1=node.childNodes->begin();
		while(iter1!=node.childNodes->end())
		{
			ShowXML(**iter1); 
			iter1++;
		} 
	}else{
		cout<<node.value ;
	} 
	if(node.name!=string()){
		cout<<"</"<<node.name<<">"<<endl; 
	}
}

void XmlNode::ShowAttr()const
{
	if(attribute!=NULL)
	{
		XmlAttribute::Iterator iter=attribute->begin();
		while(iter!=attribute->end())
		{
			cout<<BLANK<<(*iter).first<<"=\""<<(*iter).second<<"\"" ;
			iter++;
		}
	}
}

void XmlNode::SelectNodes(const string& url,vector<XmlNode*>& outList)
{
	 if(url==string())
	 {
		 return ;
	 }
	 const string tmpUrl=StringHelper::ToLower(url); 
	 vector<string> vect;
	 StringHelper::Split(tmpUrl,'/',vect);  
	 if(vect.size()==0)
	 {
		return ;
	 }

	 vector<XmlNode*> nodeList; 
	 childNodes->SelectNode(nodeList,vect[0]);
	 vector<XmlNode*> outNodeList; 

	 for(size_t i=1;i<vect.size();i++)
	 {
		 if(i&1){
			 outNodeList.clear();
			 SelectNodes(nodeList,outNodeList,vect[i]);
		 }else{
			 nodeList.clear();
			 SelectNodes(outNodeList,nodeList,vect[i]);
		 }
	 }

	 if(vect.size()&1)
	 {
		outNodeList=nodeList;
	 }
	 for(size_t i=0;i<outNodeList.size();i++)
	 {
		 outList.push_back(outNodeList[i]);
	 } 
	 nodeList.clear();
	 outNodeList.clear();
}

void XmlNode::SelectNodes(vector<XmlNode*>& inVect,vector<XmlNode*>& outVect,const string& name)
{ 
	for(int i=inVect.size()-1;i>=0;i--)
	{ 
		if(inVect[i]->childNodes==NULL|| inVect[i]->ChildCount()==0)
		{
			continue;
		}
		XmlNode::Iterator iter=inVect[i]->begin();
		while(iter!=inVect[i]->end())
		{
			if((*(iter._Ptr))->name==name)
			{
				outVect.push_back(*iter);
			}
			iter++;
		}
	}
}

XmlNode* XmlNode::SelectSingleNode(const string& url)
{
	vector<XmlNode*> outList;
	SelectNodes(url,outList);
	if(outList.size()>0)
	{
		return outList[0];
	}
	return NULL;
}

int XmlNode::Sign() const
{
	return sign;
}

bool XmlNode::operator==(const XmlNode& node)
{
	return sign==node.sign;
}

bool XmlNode::operator!=(const XmlNode& node)
{
	return !operator==(node);
}

void XmlNode::InitNode()//ʼڵ
{
	value=string();
	childNodes=NULL;
	attribute=NULL;
	sign=++SEQUENCE_SIGN; 
}

//Բѯ
XmlNode* XmlNode::SelectSingleNodeByAttr(const string& attrName,const string& attrValue)
{
	return SelectSingleNodeByAttr(attrName,attrValue,this);
}

//Բѯ--öаѯ
XmlNode* XmlNode::SelectSingleNodeByAttr(const string& attrName,const string& attrValue,XmlNode* node)
{  
	if(node==NULL)
		return NULL; 
	if(node->attribute!=NULL && (*node->attribute)[attrName]==attrValue)
	{ 
		return node;
	} 

	//ǰڵֱӽڵ
	for(int i=node->ChildCount()-1;i>=0;i--)
	{ 
		XmlAttribute* attr=(*node->childNodes)[i]->attribute;
		if(attr!=NULL && (*attr)[attrName] ==attrValue)
		{
			return (*node->childNodes)[i];
		} 
	}

	queue<XmlNode*> list; 
	for(int i=node->ChildCount()-1;i>=0;i--)
	{ 
		list.push((*node->childNodes)[i]); 
	}

	while(list.size()>0)
	{
		XmlNode* tmpNode=list.front();
		if(tmpNode->attribute!=NULL && (*tmpNode->attribute)[attrName]==attrValue)
		{ 
			return tmpNode;
		}
		for(int i=tmpNode->ChildCount()-1;i>=0;i--)
		{
			list.push((*tmpNode->childNodes)[i]); 
		}
		list.pop();
	} 
	return NULL;
}

XmlNode::Iterator XmlNode::begin()
{
	if(childNodes==NULL)	
		childNodes=new XmlNodeList();
	return childNodes->begin();
}
XmlNode::Iterator XmlNode::end()
{
	if(childNodes==NULL)	
		childNodes=new XmlNodeList();
	return childNodes->end();
}
 
#endif